'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var optimizely = require('@optimizely/optimizely-sdk');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

/**
 * Copyright 2018-2019, Optimizely
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
function find(arr, pred) {
    for (var i = 0; i < arr.length; i++) {
        if (pred(arr[i])) {
            return arr[i];
        }
    }
    return undefined;
}

var ProvidedDatafileLoader = /** @class */ (function () {
    function ProvidedDatafileLoader(config) {
        this.datafile = config.datafile;
    }
    ProvidedDatafileLoader.prototype.load = function () {
        return this.datafile;
    };
    return ProvidedDatafileLoader;
}());
var FetchUrlDatafileLoader = /** @class */ (function () {
    function FetchUrlDatafileLoader(config) {
        this.sdkKey = config.sdkKey;
        this.localStorageKey = config.localStorageKey || 'optly_fs_datafile';
    }
    FetchUrlDatafileLoader.prototype.load = function () {
        var _this = this;
        var cacheResult = this.getFromCache();
        var freshDatafileFetch = this.fetchDatafile().then(function (datafile) {
            _this.saveToCache(datafile);
            return datafile;
        });
        if (cacheResult && this.shouldUseCache(cacheResult)) {
            return cacheResult.datafile;
        }
        return freshDatafileFetch;
    };
    FetchUrlDatafileLoader.prototype.saveToCache = function (datafileToSave) {
        var _this = this;
        if (typeof window !== 'undefined') {
            var cacheEntry_1 = {
                datafile: datafileToSave,
                metadata: {
                    timestampCached: Date.now(),
                },
            };
            // use setTimeout as to not block on a potentially expensive JSON.stringify
            setTimeout(function () {
                window.localStorage.setItem(_this.localStorageKey, JSON.stringify(cacheEntry_1));
            }, 0);
        }
    };
    FetchUrlDatafileLoader.prototype.shouldUseCache = function (cacheResult) {
        return (Date.now() - cacheResult.metadata.timestampCached) <= FetchUrlDatafileLoader.MAX_CACHE_AGE_MS;
    };
    FetchUrlDatafileLoader.prototype.fetchDatafile = function () {
        var datafileUrl = "https://cdn.optimizely.com/datafiles/" + this.sdkKey + ".json";
        return new Promise(function (resolve, reject) {
            var req = new XMLHttpRequest();
            req.open(FetchUrlDatafileLoader.GET_METHOD, datafileUrl, true);
            req.onreadystatechange = function () {
                if (req.readyState === FetchUrlDatafileLoader.READY_STATE_COMPLETE) {
                    // TODO: Improve & add more error handling
                    if (req.status >= 400) {
                        reject('Error response fetching datafile');
                        return;
                    }
                    var datafile = void 0;
                    try {
                        datafile = JSON.parse(req.response);
                    }
                    catch (e) {
                        reject("Datafile was not valid JSON. Got: " + req.response);
                        return;
                    }
                    resolve(datafile);
                }
            };
            req.send();
        });
    };
    FetchUrlDatafileLoader.prototype.getFromCache = function () {
        if (typeof window === 'undefined') {
            return null;
        }
        var item = window.localStorage.getItem(this.localStorageKey);
        if (!item) {
            return null;
        }
        var toReturn;
        try {
            toReturn = JSON.parse(item);
        }
        catch (e) {
            toReturn = null;
        }
        return toReturn;
    };
    // 1 week in ms = 7 days * 24 hours * 60 minutes * 60 seconds * 1000 ms
    FetchUrlDatafileLoader.MAX_CACHE_AGE_MS = 7 * 24 * 60 * 60 * 1000;
    FetchUrlDatafileLoader.GET_METHOD = 'GET';
    FetchUrlDatafileLoader.READY_STATE_COMPLETE = 4;
    return FetchUrlDatafileLoader;
}());

var Resource = /** @class */ (function () {
    function Resource(loader) {
        this.loader = loader;
        this._hasLoaded = false;
        this.promise = this.load();
    }
    Object.defineProperty(Resource.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Resource.prototype, "hasLoaded", {
        get: function () {
            return this._hasLoaded;
        },
        enumerable: true,
        configurable: true
    });
    Resource.prototype.updateStateFromLoadResult = function (value) {
        this._value = value;
        this._hasLoaded = true;
    };
    Resource.prototype.load = function () {
        var _this = this;
        var maybeValue = this.loader.load();
        // TODO: test does this work with polyfilled promise?
        if (maybeValue instanceof Promise) {
            return maybeValue.then(function (value) {
                _this.updateStateFromLoadResult(value);
                return value;
            });
        }
        this.updateStateFromLoadResult(maybeValue);
        return Promise.resolve(maybeValue);
    };
    return Resource;
}());

var NoopNotificationCenter = /** @class */ (function () {
    function NoopNotificationCenter() {
    }
    NoopNotificationCenter.prototype.addNotificationListener = function (notificationType, callback) {
        console.warn('NotificationCenter must be setup after optimizely is initialized. (eg `await optimizely.onReady()`)');
        return -1;
    };
    NoopNotificationCenter.prototype.removeNotificationListener = function (listenerId) {
        console.warn('NotificationCenter must be setup after optimizely is initialized. (eg `await optimizely.onReady()`)');
        return false;
    };
    NoopNotificationCenter.prototype.clearAllNotificationListeners = function () {
    };
    NoopNotificationCenter.prototype.clearNotificationListeners = function (notificationType) {
    };
    return NoopNotificationCenter;
}());

/**
 * @export
 * @class OptimizelySDKWrapper
 * @implements {IOptimizelySDKWrapper}
 */
var OptimizelySDKWrapper = /** @class */ (function () {
    /**
     * Creates an instance of OptimizelySDKWrapper.
     * @param {OptimizelySDKWrapperConfig} [config={}]
     * @memberof OptimizelySDKWrapper
     */
    function OptimizelySDKWrapper(config) {
        if (config === void 0) { config = {}; }
        var _this = this;
        this.notificationCenter = new NoopNotificationCenter();
        this.initialConfig = config;
        this.isInitialized = false;
        this.datafile = null;
        this.trackEventQueue = [];
        this.datafileResource = this.setupDatafileResource(config);
        if (this.datafileResource.hasLoaded) {
            this.onInitialized();
            this.initializingPromise = Promise.resolve();
        }
        else {
            this.initializingPromise = this.datafileResource.promise.then(function () {
                _this.onInitialized();
            });
        }
    }
    /**
     * onReady happens when the datafile and attributes are fully loaded
     * Returns a promise where the resolved value is a boolean indicating whether
     * the optimizely instance has been initialized.  This only is false when
     * you supply a timeout
  
     * @param {{ timeout?: number }} [config={}]
     * @returns {Promise<boolean>}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.onReady = function (config) {
        var _this = this;
        if (config === void 0) { config = {}; }
        var timeoutId;
        if (config.timeout == null) {
            return this.initializingPromise.then(function () { return true; }, function (reason) { return false; });
        }
        else {
            // handle the case where its not initialized and timeout is set
            return Promise.race([
                this.initializingPromise,
                new Promise(function (resolve) {
                    timeoutId = setTimeout(function () { return resolve(); }, config.timeout);
                }),
            ]).then(function () {
                if (_this.isInitialized && timeoutId) {
                    clearTimeout(timeoutId);
                }
                return _this.isInitialized;
            });
        }
    };
    /**
     * @param {string} experimentKey
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {(string | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.activate = function (experimentKey, userId, attributes) {
        if (!this.isInitialized) {
            return null;
        }
        return this.instance.activate(experimentKey, userId, attributes);
    };
    /**
     *
     *
     * @param {string} experimentKey
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {(string | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getVariation = function (experimentKey, userId, attributes) {
        if (!this.isInitialized) {
            return null;
        }
        return this.instance.getVariation(experimentKey, userId, attributes);
    };
    /**
     * @param {string} eventKey
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @param {optimizely.EventTags} [eventTags]
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.track = function (eventKey, userId, attributes, eventTags) {
        if (!this.isInitialized) {
            this.trackEventQueue.push([eventKey, userId, attributes, eventTags]);
            return;
        }
        this.instance.track(eventKey, userId, attributes, eventTags);
    };
    /**
     * Note: in the case where the feature isnt in the datafile or the datafile hasnt been
     * loaded, this will return `false`
     *
     * @param {string} feature
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {boolean}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.isFeatureEnabled = function (feature, userId, attributes) {
        if (!this.isInitialized) {
            return false;
        }
        return this.instance.isFeatureEnabled(feature, userId, attributes);
    };
    /**
     * Get all variables for a feature, regardless of the feature being enabled/disabled
     *
     * @param {string} feature
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {VariableValuesObject}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getFeatureVariables = function (feature, userId, attributes) {
        var _this = this;
        if (!this.isInitialized) {
            return {};
        }
        var variableDefs = this.getVariableDefsForFeature(feature);
        if (!variableDefs) {
            // TODO: error
            return {};
        }
        var variableObj = {};
        variableDefs.forEach(function (_a) {
            var key = _a.key, type = _a.type;
            switch (type) {
                case 'string':
                    variableObj[key] = _this.instance.getFeatureVariableString(feature, key, userId, attributes);
                    break;
                case 'boolean':
                    variableObj[key] = _this.instance.getFeatureVariableBoolean(feature, key, userId, attributes);
                    break;
                case 'integer':
                    variableObj[key] = _this.instance.getFeatureVariableInteger(feature, key, userId, attributes);
                    break;
                case 'double':
                    variableObj[key] = _this.instance.getFeatureVariableDouble(feature, key, userId, attributes);
                    break;
            }
        });
        return variableObj;
    };
    /**
     * @param {string} feature
     * @param {string} variable
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {(string | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getFeatureVariableString = function (feature, variable, userId, attributes) {
        if (!this.isInitialized) {
            return null;
        }
        return this.instance.getFeatureVariableString(feature, variable, userId, attributes);
    };
    /**
     * @param {string} feature
     * @param {string} variable
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {(boolean | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getFeatureVariableBoolean = function (feature, variable, userId, attributes) {
        if (!this.isInitialized) {
            return null;
        }
        return this.instance.getFeatureVariableBoolean(feature, variable, userId, attributes);
    };
    /**
     * @param {string} feature
     * @param {string} variable
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {(number | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getFeatureVariableInteger = function (feature, variable, userId, attributes) {
        if (!this.isInitialized) {
            return null;
        }
        return this.instance.getFeatureVariableInteger(feature, variable, userId, attributes);
    };
    /**
     * @param {string} feature
     * @param {string} variable
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {(number | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getFeatureVariableDouble = function (feature, variable, userId, attributes) {
        if (!this.isInitialized) {
            return null;
        }
        return this.instance.getFeatureVariableDouble(feature, variable, userId, attributes);
    };
    /**
     * Get an array of all enabled features
     *
     * @param {string} userId
     * @param {UserAttributes} [attributes]
     * @returns {Array<string>}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getEnabledFeatures = function (userId, attributes) {
        if (!this.isInitialized) {
            return [];
        }
        return this.instance.getEnabledFeatures(userId, attributes);
    };
    /**
     * @param {string} experiment
     * @param {string} userId
     * @returns {(string | null)}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.getForcedVariation = function (experiment, userId) {
        return this.instance.getForcedVariation(experiment, userId);
    };
    /**
     * @param {string} experiment
     * @param {string} userId
     * @param {string} variationKey
     * @returns {boolean}
     * @memberof OptimizelySDKWrapper
     */
    OptimizelySDKWrapper.prototype.setForcedVariation = function (experiment, userId, variationKey) {
        return this.instance.setForcedVariation(experiment, userId, variationKey);
    };
    OptimizelySDKWrapper.prototype.getVariableDefsForFeature = function (feature) {
        if (!this.datafile) {
            return null;
        }
        var featureDef = find(this.datafile.featureFlags, function (entry) { return entry.key === feature; });
        if (!featureDef) {
            return null;
        }
        return featureDef.variables;
    };
    OptimizelySDKWrapper.prototype.flushTrackEventQueue = function () {
        while (this.trackEventQueue.length) {
            var args = this.trackEventQueue.shift();
            this.track.apply(this, args);
        }
    };
    OptimizelySDKWrapper.prototype.setupDatafileResource = function (config) {
        var datafileLoader;
        if (config.datafile) {
            datafileLoader = new ProvidedDatafileLoader({
                datafile: config.datafile,
            });
        }
        else if (config.sdkKey) {
            datafileLoader = new FetchUrlDatafileLoader({
                sdkKey: config.sdkKey,
            });
        }
        else {
            throw new Error('Must supply either "datafile", "sdkKey"');
        }
        return new Resource(datafileLoader);
    };
    OptimizelySDKWrapper.prototype.onInitialized = function () {
        var datafile = this.datafileResource.value;
        if (datafile) {
            this.datafile = datafile;
        }
        // can initialize check
        if (!this.datafile) {
            return;
        }
        this.isInitialized = true;
        this.instance = optimizely.createInstance(__assign({}, this.initialConfig, { datafile: this.datafile }));
        this.notificationCenter = this.instance.notificationCenter;
        // TODO: make sure this is flushed after notification listeners can be added
        this.flushTrackEventQueue();
    };
    return OptimizelySDKWrapper;
}());

/****************************************************************************
 * Copyright 2016-2018, Optimizely, Inc. and contributors                   *
 *                                                                          *
 * Licensed under the Apache License, Version 2.0 (the "License");          *
 * you may not use this file except in compliance with the License.         *
 * You may obtain a copy of the License at                                  *
 *                                                                          *
 *    http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                          *
 * Unless required by applicable law or agreed to in writing, software      *
 * distributed under the License is distributed on an "AS IS" BASIS,        *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. *
 * See the License for the specific language governing permissions and      *
 * limitations under the License.                                           *
 ***************************************************************************/

/**
 * Contains global enums used throughout the library
 */
var LOG_LEVEL = {
  NOTSET: 0,
  DEBUG: 1,
  INFO: 2,
  WARNING: 3,
  ERROR: 4,
};

var ERROR_MESSAGES = {
  EXPERIMENT_KEY_NOT_IN_DATAFILE: '%s: Experiment key %s is not in datafile.',
  FEATURE_NOT_IN_DATAFILE: '%s: Feature key %s is not in datafile.',
  IMPROPERLY_FORMATTED_EXPERIMENT: '%s: Experiment key %s is improperly formatted.',
  INVALID_ATTRIBUTES: '%s: Provided attributes are in an invalid format.',
  INVALID_BUCKETING_ID: '%s: Unable to generate hash for bucketing ID %s: %s',
  INVALID_DATAFILE: '%s: Datafile is invalid - property %s: %s',
  INVALID_DATAFILE_MALFORMED: '%s: Datafile is invalid because it is malformed.',
  INVALID_JSON: '%s: JSON object is not valid.',
  INVALID_ERROR_HANDLER: '%s: Provided "errorHandler" is in an invalid format.',
  INVALID_EVENT_DISPATCHER: '%s: Provided "eventDispatcher" is in an invalid format.',
  INVALID_EVENT_KEY: '%s: Event key %s is not in datafile.',
  INVALID_EVENT_TAGS: '%s: Provided event tags are in an invalid format.',
  INVALID_EXPERIMENT_KEY: '%s: Experiment key %s is not in datafile. It is either invalid, paused, or archived.',
  INVALID_EXPERIMENT_ID: '%s: Experiment ID %s is not in datafile.',
  INVALID_GROUP_ID: '%s: Group ID %s is not in datafile.',
  INVALID_LOGGER: '%s: Provided "logger" is in an invalid format.',
  INVALID_ROLLOUT_ID: '%s: Invalid rollout ID %s attached to feature %s',
  INVALID_USER_ID: '%s: Provided user ID is in an invalid format.',
  INVALID_USER_PROFILE_SERVICE: '%s: Provided user profile service instance is in an invalid format: %s.',
  JSON_SCHEMA_EXPECTED: '%s: JSON schema expected.',
  NO_DATAFILE_SPECIFIED: '%s: No datafile specified. Cannot start optimizely.',
  NO_JSON_PROVIDED: '%s: No JSON object to validate against schema.',
  NO_VARIATION_FOR_EXPERIMENT_KEY: '%s: No variation key %s defined in datafile for experiment %s.',
  UNDEFINED_ATTRIBUTE: '%s: Provided attribute: %s has an undefined value.',
  UNRECOGNIZED_ATTRIBUTE: '%s: Unrecognized attribute %s provided. Pruning before sending event to Optimizely.',
  UNABLE_TO_CAST_VALUE: '%s: Unable to cast value %s to type %s, returning null.',
  USER_NOT_IN_FORCED_VARIATION: '%s: User %s is not in the forced variation map. Cannot remove their forced variation.',
  USER_PROFILE_LOOKUP_ERROR: '%s: Error while looking up user profile for user ID "%s": %s.',
  USER_PROFILE_SAVE_ERROR: '%s: Error while saving user profile for user ID "%s": %s.',
  VARIABLE_KEY_NOT_IN_DATAFILE: '%s: Variable with key "%s" associated with feature with key "%s" is not in datafile.',
  VARIATION_ID_NOT_IN_DATAFILE: '%s: No variation ID %s defined in datafile for experiment %s.',
  VARIATION_ID_NOT_IN_DATAFILE_NO_EXPERIMENT: '%s: Variation ID %s is not in the datafile.',
  INVALID_INPUT_FORMAT: '%s: Provided %s is in an invalid format.',
  INVALID_DATAFILE_VERSION: '%s: This version of the JavaScript SDK does not support the given datafile version: %s',
  INVALID_VARIATION_KEY: '%s: Provided variation key is in an invalid format.',
};

var LOG_MESSAGES = {
  ACTIVATE_USER: '%s: Activating user %s in experiment %s.',
  DISPATCH_CONVERSION_EVENT: '%s: Dispatching conversion event to URL %s with params %s.',
  DISPATCH_IMPRESSION_EVENT: '%s: Dispatching impression event to URL %s with params %s.',
  DEPRECATED_EVENT_VALUE: '%s: Event value is deprecated in %s call.',
  EVENT_NOT_ASSOCIATED_WITH_EXPERIMENTS: '%s: Event %s is not associated with any running experiments.',
  EXPERIMENT_NOT_RUNNING: '%s: Experiment %s is not running.',
  FEATURE_ENABLED_FOR_USER: '%s: Feature %s is enabled for user %s.',
  FEATURE_NOT_ENABLED_FOR_USER: '%s: Feature %s is not enabled for user %s.',
  FEATURE_HAS_NO_EXPERIMENTS: '%s: Feature %s is not attached to any experiments.',
  FAILED_TO_PARSE_VALUE: '%s: Failed to parse event value "%s" from event tags.',
  FAILED_TO_PARSE_REVENUE: '%s: Failed to parse revenue value "%s" from event tags.',
  FORCED_BUCKETING_FAILED: '%s: Variation key %s is not in datafile. Not activating user %s.',
  INVALID_OBJECT: '%s: Optimizely object is not valid. Failing %s.',
  INVALID_CLIENT_ENGINE: '%s: Invalid client engine passed: %s. Defaulting to node-sdk.',
  INVALID_VARIATION_ID: '%s: Bucketed into an invalid variation ID. Returning null.',
  NOTIFICATION_LISTENER_EXCEPTION: '%s: Notification listener for (%s) threw exception: %s',
  NO_ROLLOUT_EXISTS: '%s: There is no rollout of feature %s.',
  NOT_ACTIVATING_USER: '%s: Not activating user %s for experiment %s.',
  NOT_TRACKING_USER: '%s: Not tracking user %s.',
  NOT_TRACKING_USER_FOR_EXPERIMENT: '%s: Not tracking user %s for experiment %s.',
  PARSED_REVENUE_VALUE: '%s: Parsed revenue value "%s" from event tags.',
  PARSED_NUMERIC_VALUE: '%s: Parsed event value "%s" from event tags.',
  RETURNING_STORED_VARIATION: '%s: Returning previously activated variation "%s" of experiment "%s" for user "%s" from user profile.',
  ROLLOUT_HAS_NO_EXPERIMENTS: '%s: Rollout of feature %s has no experiments',
  SAVED_VARIATION: '%s: Saved variation "%s" of experiment "%s" for user "%s".',
  SAVED_VARIATION_NOT_FOUND: '%s: User %s was previously bucketed into variation with ID %s for experiment %s, but no matching variation was found.',
  SHOULD_NOT_DISPATCH_ACTIVATE: '%s: Experiment %s is in "Launched" state. Not activating user.',
  SHOULD_NOT_DISPATCH_TRACK: '%s: Experiment %s is in "Launched" state. Not tracking user for it.',
  SKIPPING_JSON_VALIDATION: '%s: Skipping JSON schema validation.',
  TRACK_EVENT: '%s: Tracking event %s for user %s.',
  USER_ASSIGNED_TO_VARIATION_BUCKET: '%s: Assigned variation bucket %s to user %s.',
  USER_ASSIGNED_TO_EXPERIMENT_BUCKET: '%s: Assigned experiment bucket %s to user %s.',
  USER_BUCKETED_INTO_EXPERIMENT_IN_GROUP: '%s: User %s is in experiment %s of group %s.',
  USER_BUCKETED_INTO_TARGETING_RULE: '%s: User %s bucketed into targeting rule %s.',
  USER_IN_FEATURE_EXPERIMENT: '%s: User %s is in variation %s of experiment %s on the feature %s.',
  USER_IN_ROLLOUT: '%s: User %s is in rollout of feature %s.',
  USER_BUCKETED_INTO_EVERYONE_TARGETING_RULE: '%s: User %s bucketed into everyone targeting rule.',
  USER_NOT_BUCKETED_INTO_EVERYONE_TARGETING_RULE: '%s: User %s not bucketed into everyone targeting rule due to traffic allocation.',
  USER_NOT_BUCKETED_INTO_EXPERIMENT_IN_GROUP: '%s: User %s is not in experiment %s of group %s.',
  USER_NOT_BUCKETED_INTO_ANY_EXPERIMENT_IN_GROUP: '%s: User %s is not in any experiment of group %s.',
  USER_NOT_BUCKETED_INTO_TARGETING_RULE: '%s User %s not bucketed into targeting rule %s due to traffic allocation. Trying everyone rule.',
  USER_NOT_IN_FEATURE_EXPERIMENT: '%s: User %s is not in any experiment on the feature %s.',
  USER_NOT_IN_ROLLOUT: '%s: User %s is not in rollout of feature %s.',
  USER_FORCED_IN_VARIATION: '%s: User %s is forced in variation %s.',
  USER_MAPPED_TO_FORCED_VARIATION: '%s: Set variation %s for experiment %s and user %s in the forced variation map.',
  USER_DOESNT_MEET_CONDITIONS_FOR_TARGETING_RULE: '%s: User %s does not meet conditions for targeting rule %s.',
  USER_MEETS_CONDITIONS_FOR_TARGETING_RULE: '%s: User %s meets conditions for targeting rule %s.',
  USER_HAS_VARIATION: '%s: User %s is in variation %s of experiment %s.',
  USER_HAS_FORCED_VARIATION: '%s: Variation %s is mapped to experiment %s and user %s in the forced variation map.',
  USER_HAS_NO_VARIATION: '%s: User %s is in no variation of experiment %s.',
  USER_HAS_NO_FORCED_VARIATION: '%s: User %s is not in the forced variation map.',
  USER_HAS_NO_FORCED_VARIATION_FOR_EXPERIMENT: '%s: No experiment %s mapped to user %s in the forced variation map.',
  USER_NOT_IN_ANY_EXPERIMENT: '%s: User %s is not in any experiment of group %s.',
  USER_NOT_IN_EXPERIMENT: '%s: User %s does not meet conditions to be in experiment %s.',
  USER_RECEIVED_DEFAULT_VARIABLE_VALUE: '%s: User "%s" is not in any variation or rollout rule. Returning default value for variable "%s" of feature flag "%s".',
  USER_RECEIVED_VARIABLE_VALUE: '%s: Value for variable "%s" of feature flag "%s" is %s for user "%s"',
  VALID_DATAFILE: '%s: Datafile is valid.',
  VALID_USER_PROFILE_SERVICE: '%s: Valid user profile service provided.',
  VARIATION_REMOVED_FOR_USER: '%s: Variation mapped to experiment %s has been removed for user %s.',
  VARIABLE_REQUESTED_WITH_WRONG_TYPE: '%s: Requested variable type "%s", but variable is of type "%s". Use correct API to retrieve value. Returning None.',
  VALID_BUCKETING_ID: '%s: BucketingId is valid: "%s"',
  BUCKETING_ID_NOT_STRING: '%s: BucketingID attribute is not a string. Defaulted to userId',
};

var RESERVED_EVENT_KEYWORDS = {
  REVENUE: 'revenue',
  VALUE: 'value',
};

var CONTROL_ATTRIBUTES = {
  BOT_FILTERING: '$opt_bot_filtering',
  BUCKETING_ID: '$opt_bucketing_id',
  STICKY_BUCKETING_KEY: '$opt_experiment_bucket_map',
  USER_AGENT: '$opt_user_agent',
};

var JAVASCRIPT_CLIENT_ENGINE = 'javascript-sdk';
var NODE_CLIENT_ENGINE = 'node-sdk';
var NODE_CLIENT_VERSION = '3.0.0-rc2';

/*
 * Notification types for use with NotificationCenter
 * Format is EVENT: <list of parameters to callback>
 *
 * SDK consumers can use these to register callbacks with the notification center.
 *
 *  ACTIVATE: An impression event will be sent to Optimizely
 *  Callbacks will receive an object argument with the following properties:
 *    - experiment {Object}
 *    - userId {string}
 *    - attributes {Object|undefined}
 *    - variation {Object}
 *    - logEvent {Object}
 *
 *  TRACK: A conversion event will be sent to Optimizely
 *  Callbacks will receive the an object argument with the following properties:
 *    - eventKey {string}
 *    - userId {string}
 *    - attributes {Object|undefined}
 *    - eventTags {Object|undefined}
 *    - logEvent {Object}
 */
var NOTIFICATION_TYPES = {
  ACTIVATE: 'ACTIVATE:experiment, user_id,attributes, variation, event',
  TRACK: 'TRACK:event_key, user_id, attributes, event_tags, event',
};

/*
 * Represents the source of a decision for feature management. When a feature
 * is accessed through isFeatureEnabled or getVariableValue APIs, the decision
 * source is used to decide whether to dispatch an impression event to
 * Optimizely.
 */
var DECISION_SOURCES = {
  EXPERIMENT: 'experiment',
  ROLLOUT: 'rollout',
};

/*
 * Possible types of variables attached to features
 */
var FEATURE_VARIABLE_TYPES = {
  BOOLEAN: 'boolean',
  DOUBLE: 'double',
  INTEGER: 'integer',
  STRING: 'string',
};

/*
 * Supported datafile versions
 */
var DATAFILE_VERSIONS = {
  V2: '2',
  V3: '3',
  V4: '4',
};

var enums = {
	LOG_LEVEL: LOG_LEVEL,
	ERROR_MESSAGES: ERROR_MESSAGES,
	LOG_MESSAGES: LOG_MESSAGES,
	RESERVED_EVENT_KEYWORDS: RESERVED_EVENT_KEYWORDS,
	CONTROL_ATTRIBUTES: CONTROL_ATTRIBUTES,
	JAVASCRIPT_CLIENT_ENGINE: JAVASCRIPT_CLIENT_ENGINE,
	NODE_CLIENT_ENGINE: NODE_CLIENT_ENGINE,
	NODE_CLIENT_VERSION: NODE_CLIENT_VERSION,
	NOTIFICATION_TYPES: NOTIFICATION_TYPES,
	DECISION_SOURCES: DECISION_SOURCES,
	FEATURE_VARIABLE_TYPES: FEATURE_VARIABLE_TYPES,
	DATAFILE_VERSIONS: DATAFILE_VERSIONS
};

var optimizelyEnums = /*#__PURE__*/Object.freeze({
    default: enums,
    __moduleExports: enums,
    LOG_LEVEL: LOG_LEVEL,
    ERROR_MESSAGES: ERROR_MESSAGES,
    LOG_MESSAGES: LOG_MESSAGES,
    RESERVED_EVENT_KEYWORDS: RESERVED_EVENT_KEYWORDS,
    CONTROL_ATTRIBUTES: CONTROL_ATTRIBUTES,
    JAVASCRIPT_CLIENT_ENGINE: JAVASCRIPT_CLIENT_ENGINE,
    NODE_CLIENT_ENGINE: NODE_CLIENT_ENGINE,
    NODE_CLIENT_VERSION: NODE_CLIENT_VERSION,
    NOTIFICATION_TYPES: NOTIFICATION_TYPES,
    DECISION_SOURCES: DECISION_SOURCES,
    FEATURE_VARIABLE_TYPES: FEATURE_VARIABLE_TYPES,
    DATAFILE_VERSIONS: DATAFILE_VERSIONS
});

/**
 * Copyright 2018-2019, Optimizely
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
function createInstance(config) {
    return new OptimizelySDKWrapper(config);
}
var enums$1 = optimizelyEnums;

exports.createInstance = createInstance;
exports.enums = enums$1;
exports.OptimizelySDKWrapper = OptimizelySDKWrapper;
